@@ -1,6 +1,6 @@
 /* vim: set et ts=8 sw=8: */
 /*
- * Copyright 2014 Red Hat, Inc.
+ * Copyright (C) 2014 Red Hat, Inc.
  *
  * Geoclue is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free
@@ -24,15 +24,16 @@
 #include <libsoup/soup.h>
 #include <string.h>
 #include "gclue-3g.h"
-#include "gclue-modem-manager.h"
-#include "gclue-location.h"
+#include "gclue-modem.h"
+#include "geocode-glib/geocode-location.h"
 #include "gclue-mozilla.h"
 
 /**
  * SECTION:gclue-3g
- * @short_description: 3GPP-based geolocation
+ * @short_description: WiFi-based geolocation
+ * @include: gclue-glib/gclue-3g.h
  *
- * Contains functions to get the geolocation based on 3GPP cell towers.
+ * Contains functions to get the geolocation based on 3G cell towers.
  **/
 
 struct _GClue3GPrivate {
@@ -45,10 +46,7 @@ struct _GClue3GPrivate {
         GClue3GTower *tower;
 };
 
-G_DEFINE_TYPE_WITH_CODE (GClue3G,
-                         gclue_3g,
-                         GCLUE_TYPE_WEB_SOURCE,
-                         G_ADD_PRIVATE (GClue3G))
+G_DEFINE_TYPE (GClue3G, gclue_3g, GCLUE_TYPE_WEB_SOURCE)
 
 static gboolean
 gclue_3g_start (GClueLocationSource *source);
@@ -59,12 +57,12 @@ gclue_3g_create_query (GClueWebSource *web,
                        GError        **error);
 static SoupMessage *
 gclue_3g_create_submit_query (GClueWebSource  *web,
-                              GClueLocation   *location,
+                              GeocodeLocation *location,
                               GError         **error);
 static GClueAccuracyLevel
 gclue_3g_get_available_accuracy_level (GClueWebSource *web,
                                        gboolean available);
-static GClueLocation *
+static GeocodeLocation *
 gclue_3g_parse_response (GClueWebSource *web,
                          const char     *xml,
                          GError        **error);
@@ -103,7 +101,7 @@ on_is_3g_available_notify (GObject    *gobject,
                                        source);
 }
 
-static GClueLocation *
+static GeocodeLocation *
 gclue_3g_parse_response (GClueWebSource *web,
                          const char     *content,
                          GError        **error)
@@ -145,6 +143,8 @@ gclue_3g_class_init (GClue3GClass *klass)
         web_class->parse_response = gclue_3g_parse_response;
         web_class->get_available_accuracy_level =
                 gclue_3g_get_available_accuracy_level;
+
+        g_type_class_add_private (klass, sizeof (GClue3GPrivate));
 }
 
 static void
@@ -157,7 +157,7 @@ gclue_3g_init (GClue3G *source)
 
         priv->cancellable = g_cancellable_new ();
 
-        priv->modem = gclue_modem_manager_get_singleton ();
+        priv->modem = gclue_modem_get_singleton ();
         priv->threeg_notify_id =
                         g_signal_connect (priv->modem,
                                           "notify::is-3g-available",
@@ -217,7 +217,7 @@ gclue_3g_create_query (GClueWebSource *web,
 
 static SoupMessage *
 gclue_3g_create_submit_query (GClueWebSource  *web,
-                              GClueLocation   *location,
+                              GeocodeLocation *location,
                               GError         **error)
 {
         GClue3GPrivate *priv = GCLUE_3G (web)->priv;
