@@ -284,10 +284,10 @@ gclue_config_is_agent_allowed (GClueConfig     *config,
         return FALSE;
 }
 
-GClueAppPerm
-gclue_config_get_app_perm (GClueConfig     *config,
-                           const char      *desktop_id,
-                           GClueClientInfo *app_info)
+gboolean
+gclue_config_is_app_allowed (GClueConfig     *config,
+                             const char      *desktop_id,
+                             GClueClientInfo *app_info)
 {
         GClueConfigPrivate *priv = config->priv;
         GList *node;
@@ -305,29 +305,23 @@ gclue_config_get_app_perm (GClueConfig     *config,
                 }
         }
 
-        if (app_config == NULL) {
-                g_debug ("'%s' not in configuration", desktop_id);
+        if (app_config == NULL || !app_config->allowed) {
+                g_debug ("'%s' not in configuration or not allowed", desktop_id);
 
-                return GCLUE_APP_PERM_ASK_AGENT;
-        }
-
-        if (!app_config->allowed) {
-                g_debug ("'%s' disallowed by configuration", desktop_id);
-
-                return GCLUE_APP_PERM_DISALLOWED;
+                return FALSE;
         }
 
         if (app_config->num_users == 0)
-                return GCLUE_APP_PERM_ALLOWED;
+                return TRUE;
 
         uid = gclue_client_info_get_user_id (app_info);
 
         for (i = 0; i < app_config->num_users; i++) {
                 if (app_config->users[i] == uid)
-                        return GCLUE_APP_PERM_ALLOWED;
+                        return TRUE;
         }
 
-        return GCLUE_APP_PERM_DISALLOWED;
+        return FALSE;
 }
 
 gboolean
