@@ -25,20 +25,20 @@
 #include "gclue-service-client.h"
 #include "gclue-service-location.h"
 #include "gclue-locator.h"
-#include "gclue-enum-types.h"
+#include "public-api/gclue-enum-types.h"
 #include "gclue-config.h"
 
 #define DEFAULT_ACCURACY_LEVEL GCLUE_ACCURACY_LEVEL_CITY
 
 static void
-gclue_service_client_client_iface_init (GClueDBusClientIface *iface);
+gclue_service_client_client_iface_init (GClueClientIface *iface);
 static void
 gclue_service_client_initable_iface_init (GInitableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GClueServiceClient,
                          gclue_service_client,
-                         GCLUE_DBUS_TYPE_CLIENT_SKELETON,
-                         G_IMPLEMENT_INTERFACE (GCLUE_DBUS_TYPE_CLIENT,
+                         GCLUE_TYPE_CLIENT_SKELETON,
+                         G_IMPLEMENT_INTERFACE (GCLUE_TYPE_CLIENT,
                                                 gclue_service_client_client_iface_init)
                          G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
                                                 gclue_service_client_initable_iface_init));
@@ -52,8 +52,7 @@ struct _GClueServiceClientPrivate
 
         GClueServiceLocation *location;
         GClueServiceLocation *prev_location;
-        guint distance_threshold;
-        guint time_threshold;
+        guint threshold;
 
         GClueLocator *locator;
 
@@ -114,81 +113,27 @@ emit_location_updated (GClueServiceClient *client,
 }
 
 static gboolean
-distance_below_threshold (GClueServiceClient *client,
-                          GClueLocation      *location)
+below_threshold (GClueServiceClient *client,
+                 GeocodeLocation    *location)
 {
         GClueServiceClientPrivate *priv = client->priv;
-        GClueLocation *cur_location;
+        GeocodeLocation *cur_location;
         gdouble distance;
         gdouble threshold_km;
 
-        if (priv->distance_threshold == 0)
+        if (priv->threshold == 0)
                 return FALSE;
 
         g_object_get (priv->location,
                       "location", &cur_location,
                       NULL);
-        distance = geocode_location_get_distance_from
-                (GEOCODE_LOCATION (cur_location), GEOCODE_LOCATION (location));
+        distance = geocode_location_get_distance_from (cur_location,
+                                                       location);
         g_object_unref (cur_location);
 
-        threshold_km = priv->distance_threshold / 1000.0;
-        if (distance < threshold_km) {
-                g_debug ("Distance from previous location is %f km and "
-                         "below threshold of %f km.",
-                         distance, threshold_km); 
+        threshold_km = priv->threshold / 1000.0;
+        if (distance < threshold_km)
                 return TRUE;
-        }
-
-        return FALSE;
-}
-
-static gboolean
-time_below_threshold (GClueServiceClient *client,
-                      GClueLocation      *location)
-{
-        GClueServiceClientPrivate *priv = client->priv;
-        GClueLocation *cur_location;
-        gint64 cur_ts, ts;
-        guint64 diff_ts;
-
-        if (priv->time_threshold == 0)
-                return FALSE;
-
-        g_object_get (priv->location,
-                      "location", &cur_location,
-                      NULL);
-
-        cur_ts = geocode_location_get_timestamp
-                                (GEOCODE_LOCATION (cur_location));
-        ts = geocode_location_get_timestamp (GEOCODE_LOCATION (location));
-        diff_ts = ABS (ts - cur_ts);
-
-        g_object_unref (cur_location);
-
-        if (diff_ts < priv->time_threshold) {
-                g_debug ("Time difference between previous and new location"
-                         " is %" G_GUINT64_FORMAT " seconds and"
-                         " below threshold of %" G_GUINT32_FORMAT " seconds.",
-                         diff_ts, priv->time_threshold);
-                return TRUE;
-        }
-
-        return FALSE;
-}
-
-static gboolean
-below_threshold (GClueServiceClient *client,
-                 GClueLocation      *location)
-{
-        return (distance_below_threshold (client, location) ||
-                time_below_threshold (client, location));
-}
-
-static gboolean
-on_prev_location_timeout (gpointer user_data)
-{
-        g_object_unref (user_data);
 
         return FALSE;
 }
@@ -201,7 +146,7 @@ on_locator_location_changed (GObject    *gobject,
         GClueServiceClient *client = GCLUE_SERVICE_CLIENT (user_data);
         GClueServiceClientPrivate *priv = client->priv;
         GClueLocationSource *locator = GCLUE_LOCATION_SOURCE (gobject);
-        GClueLocation *location_info;
+        GeocodeLocation *location_info;
         char *path = NULL;
         const char *prev_path;
         GError *error = NULL;
@@ -218,10 +163,7 @@ on_locator_location_changed (GObject    *gobject,
                 return;
         }
 
-        if (priv->prev_location != NULL)
-                // Lets try to ensure that apps are not still accessing the
-                // last location before unrefing (and therefore destroying) it.
-                g_timeout_add_seconds (5, on_prev_location_timeout, priv->prev_location);
+        g_clear_object (&priv->prev_location);
         priv->prev_location = priv->location;
 
         path = next_location_path (client);
@@ -238,7 +180,7 @@ on_locator_location_changed (GObject    *gobject,
         else
                 prev_path = "/";
 
-        gclue_dbus_client_set_location (GCLUE_DBUS_CLIENT (client), path);
+        gclue_client_set_location (GCLUE_CLIENT (client), path);
 
         if (!emit_location_updated (client, prev_path, path, &error))
                 goto error_out;
@@ -256,7 +198,7 @@ start_client (GClueServiceClient *client, GClueAccuracyLevel accuracy_level)
 {
         GClueServiceClientPrivate *priv = client->priv;
 
-        gclue_dbus_client_set_active (GCLUE_DBUS_CLIENT (client), TRUE);
+        gclue_client_set_active (GCLUE_CLIENT (client), TRUE);
         priv->locator = gclue_locator_new (accuracy_level);
         g_signal_connect (priv->locator,
                           "notify::location",
@@ -270,7 +212,7 @@ static void
 stop_client (GClueServiceClient *client)
 {
         g_clear_object (&client->priv->locator);
-        gclue_dbus_client_set_active (GCLUE_DBUS_CLIENT (client), FALSE);
+        gclue_client_set_active (GCLUE_CLIENT (client), FALSE);
 }
 
 static void
@@ -280,12 +222,11 @@ on_agent_props_changed (GDBusProxy *agent_proxy,
                         gpointer    user_data)
 {
         GClueServiceClient *client = GCLUE_SERVICE_CLIENT (user_data);
-        GClueDBusClient *gdbus_client;
         GVariantIter *iter;
         GVariant *value;
         gchar *key;
         
-        if (g_variant_n_children (changed_properties) <= 0)
+        if (g_variant_n_children (changed_properties) < 0)
                 return;
 
         g_variant_get (changed_properties, "a{sv}", &iter);
@@ -297,9 +238,8 @@ on_agent_props_changed (GDBusProxy *agent_proxy,
                 if (strcmp (key, "MaxAccuracyLevel") != 0)
                         continue;
 
-                gdbus_client = GCLUE_DBUS_CLIENT (client);
                 config = gclue_config_get_singleton ();
-                id = gclue_dbus_client_get_desktop_id (gdbus_client);
+                id = gclue_client_get_desktop_id (GCLUE_CLIENT (client));
                 max_accuracy = g_variant_get_uint32 (value);
                 /* FIXME: We should be handling all values of max accuracy
                  *        level here, not just 0 and non-0.
@@ -308,14 +248,14 @@ on_agent_props_changed (GDBusProxy *agent_proxy,
                         GClueAccuracyLevel accuracy;
 
                         client->priv->agent_stopped = FALSE;
-                        accuracy = gclue_dbus_client_get_requested_accuracy_level
-                                (gdbus_client);
+                        accuracy = gclue_client_get_requested_accuracy_level
+                                (GCLUE_CLIENT (client));
                         accuracy = CLAMP (accuracy, 0, max_accuracy);
 
                         start_client (client, accuracy);
                         g_debug ("Re-started '%s'.", id);
                 } else if (max_accuracy == 0 &&
-                           gclue_dbus_client_get_active (gdbus_client) &&
+                           gclue_client_get_active (GCLUE_CLIENT (client)) &&
                            !gclue_config_is_system_component (config, id)) {
                         stop_client (client);
                         client->priv->agent_stopped = TRUE;
@@ -331,8 +271,6 @@ typedef struct
 {
         GClueServiceClient *client;
         GDBusMethodInvocation *invocation;
-        char *desktop_id;
-        GClueAccuracyLevel accuracy_level;
 } StartData;
 
 static void
@@ -340,20 +278,18 @@ start_data_free (StartData *data)
 {
         g_object_unref (data->client);
         g_object_unref (data->invocation);
-        g_free(data->desktop_id);
         g_slice_free (StartData, data);
 }
 
 static void
-complete_start (StartData *data)
+complete_start (StartData *data, GClueAccuracyLevel accuracy_level)
 {
-        GClueDBusClient *gdbus_client = GCLUE_DBUS_CLIENT (data->client);
-        start_client (data->client, data->accuracy_level);
+        start_client (data->client, accuracy_level);
 
-        gclue_dbus_client_complete_start (gdbus_client,
-                                          data->invocation);
+        gclue_client_complete_start (GCLUE_CLIENT (data->client),
+                                     data->invocation);
         g_debug ("'%s' started.",
-                 gclue_dbus_client_get_desktop_id (gdbus_client));
+                 gclue_client_get_desktop_id (GCLUE_CLIENT (data->client)));
         start_data_free (data);
 }
 
@@ -363,35 +299,28 @@ on_authorize_app_ready (GObject      *source_object,
                         gpointer      user_data)
 {
         StartData *data = (StartData *) user_data;
-        GClueServiceClientPrivate *priv = data->client->priv;
         GError *error = NULL;
         gboolean authorized = FALSE;
+        GClueAccuracyLevel accuracy_level;
 
+        accuracy_level = gclue_client_get_requested_accuracy_level
+                                (GCLUE_CLIENT (data->client));
         if (!gclue_agent_call_authorize_app_finish (GCLUE_AGENT (source_object),
                                                     &authorized,
-                                                    &data->accuracy_level,
+                                                    &accuracy_level,
                                                     res,
                                                     &error))
                 goto error_out;
 
         if (!authorized) {
-                guint32 uid;
-
-                uid = gclue_client_info_get_user_id (priv->client_info);
-
-                g_set_error (&error,
-                             G_DBUS_ERROR,
-                             G_DBUS_ERROR_ACCESS_DENIED,
-                             "Agent rejected '%s' for user '%u'. Please ensure "
-                             "that '%s' has installed a valid %s.desktop file.",
-                             data->desktop_id,
-                             uid,
-                             data->desktop_id,
-                             data->desktop_id);
+                g_set_error_literal (&error,
+                                     G_DBUS_ERROR,
+                                     G_DBUS_ERROR_ACCESS_DENIED,
+                                     "Access denied");
                 goto error_out;
         }
 
-        complete_start (data);
+        complete_start (data, accuracy_level);
 
         return;
 
@@ -401,78 +330,50 @@ error_out:
 }
 
 static gboolean
-gclue_service_client_handle_start (GClueDBusClient       *client,
+gclue_service_client_handle_start (GClueClient           *client,
                                    GDBusMethodInvocation *invocation)
 {
         GClueServiceClientPrivate *priv = GCLUE_SERVICE_CLIENT (client)->priv;
         GClueConfig *config;
         StartData *data;
         const char *desktop_id;
-        GClueAccuracyLevel max_accuracy;
-        GClueAppPerm app_perm;
-        guint32 uid;
+        GClueAccuracyLevel accuracy_level, max_accuracy;
 
-        if (priv->locator != NULL) {
+        if (priv->locator != NULL)
                 /* Already started */
-                gclue_dbus_client_complete_start (client, invocation);
-
                 return TRUE;
-        }
-
-        desktop_id = gclue_client_info_get_xdg_id (priv->client_info);
-        if (desktop_id == NULL)
-                /* Non-xdg app */
-                desktop_id = gclue_dbus_client_get_desktop_id (client);
 
+        desktop_id = gclue_client_get_desktop_id (client);
         if (desktop_id == NULL) {
-                g_dbus_method_invocation_return_error_literal (invocation,
-                                                               G_DBUS_ERROR,
-                                                               G_DBUS_ERROR_ACCESS_DENIED,
-                                                               "'DesktopId' property must be set");
-                return TRUE;
-        }
-
-        config = gclue_config_get_singleton ();
-        uid = gclue_client_info_get_user_id (priv->client_info);
-        app_perm = gclue_config_get_app_perm (config,
-                                              desktop_id,
-                                              priv->client_info);
-        if (app_perm == GCLUE_APP_PERM_DISALLOWED) {
                 g_dbus_method_invocation_return_error (invocation,
                                                        G_DBUS_ERROR,
                                                        G_DBUS_ERROR_ACCESS_DENIED,
-                                                       "'%s' disallowed by "
-                                                       "configuration for UID %u",
-                                                       desktop_id,
-                                                       uid);
+                                                       "'DesktopId' property must be set");
                 return TRUE;
         }
 
         data = g_slice_new (StartData);
         data->client = g_object_ref (client);
         data->invocation =  g_object_ref (invocation);
-        data->desktop_id =  g_strdup (desktop_id);
 
-        data->accuracy_level = gclue_dbus_client_get_requested_accuracy_level (client);
-        data->accuracy_level = CLAMP (data->accuracy_level,
-                                      GCLUE_ACCURACY_LEVEL_COUNTRY,
-                                      GCLUE_ACCURACY_LEVEL_EXACT);
+        config = gclue_config_get_singleton ();
+        accuracy_level = gclue_client_get_requested_accuracy_level (client);
 
         /* No agent == No authorization needed */
         if (priv->agent_proxy == NULL ||
             gclue_config_is_system_component (config, desktop_id) ||
-            app_perm == GCLUE_APP_PERM_ALLOWED) {
-                complete_start (data);
+            gclue_config_is_app_allowed (config,
+                                         desktop_id,
+                                         priv->client_info)) {
+                complete_start (data, accuracy_level);
 
                 return TRUE;
         }
 
-        if (priv->agent_proxy != NULL)
-                max_accuracy = gclue_agent_get_max_accuracy_level (priv->agent_proxy);
-        else
-                max_accuracy = GCLUE_ACCURACY_LEVEL_EXACT;
-
+        max_accuracy = gclue_agent_get_max_accuracy_level (priv->agent_proxy);
         if (max_accuracy == 0) {
+                guint32 uid = gclue_client_info_get_user_id (priv->client_info);
+
                 g_dbus_method_invocation_return_error (invocation,
                                                        G_DBUS_ERROR,
                                                        G_DBUS_ERROR_ACCESS_DENIED,
@@ -482,14 +383,11 @@ gclue_service_client_handle_start (GClueDBusClient       *client,
                 start_data_free (data);
                 return TRUE;
         }
-        g_debug ("requested accuracy level: %u. "
-                 "Max accuracy level allowed by agent: %u",
-                 data->accuracy_level, max_accuracy);
-        data->accuracy_level = CLAMP (data->accuracy_level, 0, max_accuracy);
+        accuracy_level = CLAMP (accuracy_level, 0, max_accuracy);
 
         gclue_agent_call_authorize_app (priv->agent_proxy,
                                         desktop_id,
-                                        data->accuracy_level,
+                                        accuracy_level,
                                         NULL,
                                         on_authorize_app_ready,
                                         data);
@@ -498,12 +396,12 @@ gclue_service_client_handle_start (GClueDBusClient       *client,
 }
 
 static gboolean
-gclue_service_client_handle_stop (GClueDBusClient       *client,
+gclue_service_client_handle_stop (GClueClient           *client,
                                   GDBusMethodInvocation *invocation)
 {
         stop_client (GCLUE_SERVICE_CLIENT (client));
-        gclue_dbus_client_complete_stop (client, invocation);
-        g_debug ("'%s' stopped.", gclue_dbus_client_get_desktop_id (client));
+        gclue_client_complete_stop (client, invocation);
+        g_debug ("'%s' stopped.", gclue_client_get_desktop_id (client));
 
         return TRUE;
 }
@@ -515,11 +413,9 @@ gclue_service_client_finalize (GObject *object)
 
         g_clear_pointer (&priv->path, g_free);
         g_clear_object (&priv->connection);
-        if (priv->agent_proxy != NULL)
-                g_signal_handlers_disconnect_by_func
-                                (priv->agent_proxy,
-                                 G_CALLBACK (on_agent_props_changed),
-                                 object);
+        g_signal_handlers_disconnect_by_func (priv->agent_proxy,
+                                              G_CALLBACK (on_agent_props_changed),
+                                              object);
         g_clear_object (&priv->agent_proxy);
         g_clear_object (&priv->locator);
         g_clear_object (&priv->location);
@@ -583,11 +479,10 @@ gclue_service_client_set_property (GObject      *object,
 
         case PROP_AGENT_PROXY:
                 client->priv->agent_proxy = g_value_dup_object (value);
-                if (client->priv->agent_proxy != NULL)
-                        g_signal_connect (client->priv->agent_proxy,
-                                          "g-properties-changed",
-                                          G_CALLBACK (on_agent_props_changed),
-                                          object);
+                g_signal_connect (client->priv->agent_proxy,
+                                  "g-properties-changed",
+                                  G_CALLBACK (on_agent_props_changed),
+                                  object);
                 break;
 
         default:
@@ -610,10 +505,10 @@ gclue_service_client_handle_method_call (GDBusConnection       *connection,
         GDBusInterfaceVTable *skeleton_vtable;
 
         if (!gclue_client_info_check_bus_name (priv->client_info, sender)) {
-                g_dbus_method_invocation_return_error_literal (invocation,
-                                                               G_DBUS_ERROR,
-                                                               G_DBUS_ERROR_ACCESS_DENIED,
-                                                               "Access denied");
+                g_dbus_method_invocation_return_error (invocation,
+                                                       G_DBUS_ERROR,
+                                                       G_DBUS_ERROR_ACCESS_DENIED,
+                                                       "Access denied");
                 return;
         }
 
@@ -671,7 +566,7 @@ gclue_service_client_handle_set_property (GDBusConnection *connection,
                                           GError         **error,
                                           gpointer        user_data)
 {
-        GClueDBusClient *client = GCLUE_DBUS_CLIENT (user_data);
+        GClueClient *client = GCLUE_CLIENT (user_data);
         GClueServiceClientPrivate *priv = GCLUE_SERVICE_CLIENT (client)->priv;
         GDBusInterfaceSkeletonClass *skeleton_class;
         GDBusInterfaceVTable *skeleton_vtable;
@@ -696,13 +591,8 @@ gclue_service_client_handle_set_property (GDBusConnection *connection,
                                              error,
                                              user_data);
         if (ret && strcmp (property_name, "DistanceThreshold") == 0) {
-                priv->distance_threshold = gclue_dbus_client_get_distance_threshold
-                        (client);
-                g_debug ("New distance threshold: %u", priv->distance_threshold);
-        } else if (ret && strcmp (property_name, "TimeThreshold") == 0) {
-                priv->time_threshold = gclue_dbus_client_get_time_threshold
-                        (client);
-                g_debug ("New time threshold: %u", priv->time_threshold);
+                priv->threshold = gclue_client_get_distance_threshold (client);
+                g_debug ("New distance threshold: %u", priv->threshold);
         }
 
         return ret;
@@ -780,7 +670,7 @@ gclue_service_client_class_init (GClueServiceClientClass *klass)
 }
 
 static void
-gclue_service_client_client_iface_init (GClueDBusClientIface *iface)
+gclue_service_client_client_iface_init (GClueClientIface *iface)
 {
         iface->handle_start = gclue_service_client_handle_start;
         iface->handle_stop = gclue_service_client_handle_stop;
@@ -813,8 +703,8 @@ gclue_service_client_init (GClueServiceClient *client)
         client->priv = G_TYPE_INSTANCE_GET_PRIVATE (client,
                                                     GCLUE_TYPE_SERVICE_CLIENT,
                                                     GClueServiceClientPrivate);
-        gclue_dbus_client_set_requested_accuracy_level
-                (GCLUE_DBUS_CLIENT (client), DEFAULT_ACCURACY_LEVEL);
+        gclue_client_set_requested_accuracy_level (GCLUE_CLIENT (client),
+                                                   DEFAULT_ACCURACY_LEVEL);
 }
 
 GClueServiceClient *
