@@ -67,9 +67,9 @@ gclue_wifi_create_query (GClueWebSource *source,
                          GError        **error);
 static SoupMessage *
 gclue_wifi_create_submit_query (GClueWebSource  *source,
-                                GClueLocation   *location,
+                                GeocodeLocation *location,
                                 GError         **error);
-static GClueLocation *
+static GeocodeLocation *
 gclue_wifi_parse_response (GClueWebSource *source,
                            const char     *json,
                            GError        **error);
@@ -238,32 +238,6 @@ get_bssid_from_bss (WPABSS *bss)
         return bssid;
 }
 
-static void
-add_bss_proxy (GClueWifi *wifi,
-               WPABSS    *bss)
-{
-        const char *path;
-        char *ssid;
-
-        ssid = get_ssid_from_bss (bss);
-        g_debug ("WiFi AP '%s' added.", ssid);
-        g_free (ssid);
-
-        /* There could be multiple devices being added/removed at the same time
-         * so we don't immediately call refresh but rather wait 1 second.
-         */
-        if (wifi->priv->refresh_timeout != 0)
-                g_source_remove (wifi->priv->refresh_timeout);
-        wifi->priv->refresh_timeout = g_timeout_add_seconds (1,
-                                                             on_refresh_timeout,
-                                                             wifi);
-
-        path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (bss));
-        g_hash_table_replace (wifi->priv->bss_proxies,
-                              g_strdup (path),
-                              bss);
-}
-
 static void
 on_bss_signal_notify (GObject    *gobject,
                       GParamSpec *pspec,
@@ -282,12 +256,19 @@ on_bss_signal_notify (GObject    *gobject,
                 g_free (bssid);
                 return;
         }
+        path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (bss));
 
         g_signal_handlers_disconnect_by_func (G_OBJECT (bss),
                                               on_bss_signal_notify,
                                               user_data);
-        add_bss_proxy (wifi, g_object_ref (bss));
-        path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (bss));
+        on_bss_added (WPA_INTERFACE (wifi->priv->interface),
+                      path,
+                      NULL,
+                      user_data);
+
+        g_hash_table_replace (wifi->priv->bss_proxies,
+                              g_strdup (path),
+                              bss);
         g_hash_table_remove (wifi->priv->ignored_bss_proxies, path);
 }
 
@@ -299,6 +280,7 @@ on_bss_proxy_ready (GObject      *source_object,
         GClueWifi *wifi = GCLUE_WIFI (user_data);
         WPABSS *bss;
         GError *error = NULL;
+        const char *path;
         char *ssid;
 
         bss = wpa_bss_proxy_new_for_bus_finish (res, &error);
@@ -308,6 +290,7 @@ on_bss_proxy_ready (GObject      *source_object,
 
                 return;
         }
+        path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (bss));
 
         if (gclue_mozilla_should_ignore_bss (bss)) {
                 g_object_unref (bss);
@@ -319,8 +302,8 @@ on_bss_proxy_ready (GObject      *source_object,
         g_debug ("WiFi AP '%s' added.", ssid);
         g_free (ssid);
 
+        path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (bss));
         if (wpa_bss_get_signal (bss) <= -90) {
-                const char *path;
                 char *bssid = get_bssid_from_bss (bss);
                 g_debug ("WiFi AP '%s' has very low strength (%u dBm)"
                          ", ignoring for now..",
@@ -331,14 +314,24 @@ on_bss_proxy_ready (GObject      *source_object,
                                   "notify::signal",
                                   G_CALLBACK (on_bss_signal_notify),
                                   user_data);
-                path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (bss));
                 g_hash_table_replace (wifi->priv->ignored_bss_proxies,
                                       g_strdup (path),
                                       bss);
                 return;
         }
 
-        add_bss_proxy (wifi, bss);
+
+        /* There could be multiple devices being added/removed at the same time
+         * so we don't immediately call refresh but rather wait 1 second.
+         */
+        if (wifi->priv->refresh_timeout != 0)
+                g_source_remove (wifi->priv->refresh_timeout);
+        wifi->priv->refresh_timeout = g_timeout_add_seconds (1,
+                                                             on_refresh_timeout,
+                                                             wifi);
+        g_hash_table_replace (wifi->priv->bss_proxies,
+                              g_strdup (path),
+                              bss);
 }
 
 static void
@@ -377,24 +370,22 @@ connect_bss_signals (GClueWifi *wifi)
         if (priv->bss_added_id != 0)
                 return;
         if (priv->interface == NULL) {
-                gclue_web_source_refresh (GCLUE_WEB_SOURCE (wifi));
+                if (wifi->priv->accuracy_level == GCLUE_ACCURACY_LEVEL_CITY)
+                        gclue_web_source_refresh (GCLUE_WEB_SOURCE (wifi));
 
                 return;
         }
 
         priv->bss_added_id = g_signal_connect (priv->interface,
-                                               "bss-added",
-                                               G_CALLBACK (on_bss_added),
-                                               wifi);
+                                              "bss-added",
+                                              G_CALLBACK (on_bss_added),
+                                              wifi);
         priv->bss_removed_id = g_signal_connect (priv->interface,
                                                 "bss-removed",
                                                 G_CALLBACK (on_bss_removed),
                                                 wifi);
 
         bss_list = wpa_interface_get_bsss (WPA_INTERFACE (priv->interface));
-        if (bss_list == NULL)
-                return;
-
         for (i = 0; bss_list[i] != NULL; i++)
                 on_bss_added (WPA_INTERFACE (priv->interface),
                               bss_list[i],
@@ -631,8 +622,6 @@ gclue_wifi_get_singleton (GClueAccuracyLevel level)
         guint i;
 
         g_return_val_if_fail (level >= GCLUE_ACCURACY_LEVEL_CITY, NULL);
-        if (level == GCLUE_ACCURACY_LEVEL_NEIGHBORHOOD)
-                level = GCLUE_ACCURACY_LEVEL_CITY;
 
         i = (level == GCLUE_ACCURACY_LEVEL_CITY)? 0 : 1;
         if (wifi[i] == NULL) {
@@ -683,7 +672,7 @@ gclue_wifi_create_query (GClueWebSource *source,
         return gclue_mozilla_create_query (bss_list, NULL, error);
 }
 
-static GClueLocation *
+static GeocodeLocation *
 gclue_wifi_parse_response (GClueWebSource *source,
                            const char     *json,
                            GError        **error)
@@ -693,7 +682,7 @@ gclue_wifi_parse_response (GClueWebSource *source,
 
 static SoupMessage *
 gclue_wifi_create_submit_query (GClueWebSource  *source,
-                                GClueLocation   *location,
+                                GeocodeLocation *location,
                                 GError         **error)
 {
         GList *bss_list; /* As in Access Points */
